<?

/*
 * knitPro - a web application for generating knitting patterns
 * Copyright (C) 2004 - 2005  Cat Mazza & Eric St. Onge
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Send questions or comments to knitPro@microrevolt.org.
 */


// load a the rospdf class. knitPro uses this class
// to generate the kniting pattern as a pdf.

// rospdf requires that PHP has the GD library installed. 
include('./lib/rospdf009e/class.pdf.php');


// generateKnitProPDF() is the function that does all of the work for
// the application. Its sole argument is a hashed array of the image
// that will be turned into a PDF.

// This function uses several constants that are defined in index.php,
// including PAGE_HEIGHT, PAGE_WIDTH, BOX_SIZE, and a few others.

function generateKnitProPDF($img)
{
	
	// Get the width of the image
	$width = ImagesX($img);
	
	// Get the height of the image
	$height = ImagesY($img);

    // initialize the "scale factor" for the image.
    // if the image is too big to be displayed in the dimensions
    // required, it will be scaled down to fit.
    // knitPro will not ever scale an image up to fit, since that
    // could result in distorted images.
	$scale = 1;

    // figure out if the aspect ratio of the image.
    // we change the dimensions of the page accordingly
    // if the image is in landscape or portrait aspect ratios.

    // if the image is more wide than tall, the pattern should
    // be generated using landscape aspect ratio
	if ($width > $height) {
	
        // get the preset width and height of the page (in point)
        // since the defaults are for portrait mode
        // and this image is a landscape, swap the parameters.
        $PAGE_WIDTH = PAGE_HEIGHT;
		$PAGE_HEIGHT = PAGE_WIDTH;
		
		// if there are more vertical pixels in the image
		// than the number of boxes we have available,
		// figure out how much we have to scale the image down.
		if (($width * $scale) > MAX_BOXES_VERTICAL) 
			$scale = (MAX_BOXES_VERTICAL / $width);
		
		// we still might have to scale the image in the other
		// direction, too.
		if (($height * $scale) > MAX_BOXES_HORIZONTAL)
			$scale = MAX_BOXES_HORIZONTAL / $height;
		
			
	}
	else {
	
	    // do the same as above, but for images that are in
	    // either square or portrait aspect ratios.
		$PAGE_WIDTH = PAGE_WIDTH; // 8.5in
		$PAGE_HEIGHT = PAGE_HEIGHT; // 11in	

		if (($height * $scale) > MAX_BOXES_VERTICAL) 
			$scale = MAX_BOXES_VERTICAL / $height;

		if (($width * $scale) > MAX_BOXES_HORIZONTAL)
			$scale = MAX_BOXES_HORIZONTAL / $width;

	}
	
	// the number of boxes must be a whole number,
	// so figure out the number of boxes to use
	// by scaling down.
	$horizontalPixels = floor($width * $scale);
	$verticalPixels = floor($height * $scale);
	
	// convert the image to RGB mode so that we can reference
	// the colors correctly for all color types.
	$newImg = ImageCreateTrueColor($horizontalPixels, $verticalPixels);

    // define the first color to be black. This turns all transparent pixels black.
    // transparency could probably be handled better...
	ImageColorTransparent($img, 0);
 
    // copy the old image into the new RGB image reference
    // and resize the image as necessary
	ImageCopyResampled($newImg, $img,
							0, 0,
							0, 0,
							$horizontalPixels, $verticalPixels,
							$width, $height);
							
	
    // since the image has now been resized, reset the original image width and height
    // to the new image dimensions
	$width = $horizontalPixels;
	$height = $verticalPixels;
	$img = $newImg;
	
    // get the size of the box as defined in index.php
	$squareSize = BOX_SIZE;
	
	// set the lenght of the line (in pt) from which the guidelines
	// should extend from the edge of the pattern's gride
	$lineExtension = 16;
	
	// get the size of the grid in point
	$gridHorizontalSize = $squareSize * $horizontalPixels;
	$gridVerticalSize = $squareSize * $verticalPixels;
	
	// set the origin point for the displayed pattern.
	// this centers the grid in the middle of the page
	// in both directions.
	$baseX = ($PAGE_WIDTH - $gridHorizontalSize) / 2;
	$baseY = ($PAGE_HEIGHT - $gridVerticalSize) / 2;

    // create a new PDF object so that we can start drawing the grid
	$pdf = new Cpdf(array(0,0, $PAGE_WIDTH, $PAGE_HEIGHT));
	
	// select a default font of Helvetica
	$pdf->selectFont('lib/rospdf009e/fonts/Helvetica');
	
	// draw a URL to microrevolt.org in the lower left corner of the page.
	$pdf->addText(36, 36, 10, "http://www.microrevolt.org/");

    // use a standard line style 
	$pdf->setLineStyle(1);
	
	// this is where the bulk of the work occurs.
	
	// go through each pixel of the image and draw an element in the grid for it.
	for ($x = 0; $x < $width; $x++) {
	
		for ($y = 0; $y < $height; $y++) {
		
		    // get the color at this pixel in the resampled image
			$colorIndex = ImageColorAt($img, $x, $height - 1 - $y);
			$color = ImageColorsForIndex($img, $colorIndex);

            // change back to indexed color mode (since that's all the PDF generator knows)
			$pdf->setColor($color["red"] / 255, $color["green"] / 255, $color["blue"] / 255);
			
			// generate the rectangle
			$pdf->filledRectangle(	$baseX + ($squareSize * $x), 
									$baseY + ($squareSize * $y),
									$squareSize, $squareSize);
		}
	}

    // now draw lines in the image to show the grid
	// 1pt straight line  in a light blue.
	$pdf->setLineStyle(1);
	$pdf->setStrokeColor(0.2, 0.8, 1.0);
			
	// for horizontal pixels, draw vertical lines
	for ($t = 0; $t <= $horizontalPixels; $t++) {        	
		$pdf->line( 	$baseX + ($t * $squareSize),	
							$baseY - $lineExtension,
						$baseX + ($t * $squareSize),	
							$baseY + $gridVerticalSize + $lineExtension);
							
		/*if ( ($t + 1) % 2 == 0) {
			$pdf->addText(, , 8, "<b>" . ($t + 1) . "</b>");
		}*/
	}

	// for vertical pixels, draw horizontal lines
	for ($t = 0; $t <= $verticalPixels; $t++) {
		$pdf->line( 	$baseX - $lineExtension, 						
							$baseY + ($t * $squareSize),
						$baseX + $gridHorizontalSize + $lineExtension, 	
							$baseY + ($t * $squareSize));
	}
	
	
	// draw line numbers for every other grid line
	
	// change font to black
	$pdf->setColor(0, 0, 0);
	
	// horizontal grid numbers
	$hNumPosition = $baseX - $lineExtension + 2;
    for ($i = 1; $i < $verticalPixels; $i += 2) {
		$vNumPosition = $baseY + ($i * $squareSize) + 2;
		$pdf->addText($hNumPosition, $vNumPosition, 6, "<b>" . ($i + 1). "</b>");
	}
	
	// vertical grid numbers
	$vNumPosition = $baseY - $lineExtension + 2;
	for ($i = 1; $i < $horizontalPixels; $i += 2) {
		$hNumPosition = $baseX + ($i * $squareSize) + 2;
		$pdf->addText($hNumPosition, $vNumPosition, 6, "<b>" . ($i + 1). "</b>");
	}
	
	
	// if the option to save generated PDFs to disk has been
	// enabled, do that now.
	if (ENABLE_PROCESSED_FILE_STORAGE == "true") {
	
	   // get the PDF file contents.
	   $file_contents = $pdf->output();
	   
	   // make up a file name for this file.
	   // e.g., /home/eric/processed_knitPro_files/2005_11_28_11_00_31.myfile.gif.pdf
	   // this should always be unique.
	   $processed_file_name = PROCESSED_FILE_STORAGE_LOCATION 
	       . date("Y_m_d_H_i_s") . "." . $_FILES[UPLOAD_NAME]["name"] 
	       . ".pdf";
	
	   // try to get a file handle for the new file.
       if (!$handle = fopen($processed_file_name, "w")) {
          echo "Could not open new processed file ($processed_file_name)";
          exit();
        }
        else {
            // write the file to disk.
            if (fwrite($handle, $file_contents) === FALSE) {
                echo "Could not write processed file ($processed_file_name)";
                exit();
            }
            
            // close the file handle.
            fclose($handle);
        }
	}

    // send the generated pdf to the user's browser
	$pdf->stream();
}


?>